import { world, system, ItemStack, ItemTypes, ItemLockMode } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";
import { feelings, showFeelings } from "./feelings.js"
import { showHelp } from "./cfConfigFunctions.js"
import { getPlayerByName } from "./playerData.js";
import { timeline } from "./cinematic.js";
import { interactable_entities } from "./guides.js"


const beforeEvents = world.beforeEvents;
const afterEvents = world.afterEvents;
const overworld = world.getDimension("overworld");
let gamestarted = false;
let participatedPlayers = [];
let eliminatedPlayers = [];
let startedmapPlayers = [];
const startguessbutton = [158, 72, 137];
const platform = [165, 72, 141];
const numblock = [177, 72, 140];
let guessStarted = false;
let random = 0;
let distinctNumbers = null;
let score = 0;
let round = 0;
let roundId = null;
let dunkPlayer = null;
let timePeriod = "night";
let dunksSpawned = 0;
const coasterstart = [-34, 76, 68];
const gamelobby = [-79, 108, 12];
const mainlobby = [-70, 108, 12];
const joinbutton = [-67, 109, 12];
const startspleefbutton = [-82, 109, 12];
const startdunkbutton = [75, 72, 99];
const dunksnowball = [67, 71, 98];
const leavebutton = [-75, 109, 12];
const spawn = [-74, 102, 12];
const groundbutton = [-70, 109, 4];
const ground = [-58, 70, 20];
const snowballrestock1 = [-83, -57];
const snowballrestock2 = [-69, -71];
const snowballrestock3 = [-83, -85];
const snowballrestock4 = [-97, -71];
const bowarrow = [164, 124];
const startmap = [29, 48, 20];
const listItem = "ww:map_menu";
const guide = "ww:guide";
const places = [
    {
        "name": "Map Spawn",
        "coords": [40, 72, 46],
        "icon": "textures/items/spawn.png"
    },
    {
        "name": "Casino Fort",
        "coords": [106, 71, 124],
        "icon": "textures/items/bells.png"
    },
    {
        "name": "Jingle Roller Coaster",
        "coords": [-23, 71, 52],
        "icon": "textures/items/coaster.png"
    },
    {
        "name": "Snowball Spleef Arena",
        "coords": [-70, 108, 12],
        "icon": "textures/items/goldshovel.png"
    },
    {
        "name": "Snowball Fight Globe Arena",
        "coords": [-83, 76,-74],
        "icon": "textures/items/snow_ball.png"
    },
    {
        "name": "Jumping Castle",
        "coords": [-50, 71, -19],
        "icon": "textures/items/edible_candy_cane.png"
    },
    {
        "name": "Gifts Arena",
        "coords": [70, 70, 80],
        "icon": "textures/blocks/blue_gift.png"
    },
    {
        "name": "Elf Dunk Tank",
        "coords": [71, 71, 96],
        "icon": "textures/items/target.png"
    },
    {
        "name": "Ice Skating Arena",
        "coords": [-18, 71, 6],
        "icon": "textures/items/ice_block.png"
    },
    {
        "name": "Candy Store",
        "coords": [88, 72, 77],
        "icon": "textures/items/cookies/gingerbread_cookie.png"
    },
    {
        "name": "Garments Store",
        "coords": [-14, 71, -5],
        "icon": "textures/items/garments.png"
    }
];
const num = {
    '0': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]],
        [numblock[0],numblock[1]+3,numblock[2]+2],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '1': [
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+3,numblock[2]],
        [numblock[0],numblock[1]+3,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+1,numblock[2]+1],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '2': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]+2],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '3': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]+2],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '4': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+3,numblock[2]],
        [numblock[0],numblock[1]+3,numblock[2]+2],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '5': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '6': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '7': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]+2],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '8': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]],
        [numblock[0],numblock[1]+3,numblock[2]+2],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ],
    '9': [
        [numblock[0],numblock[1]+4,numblock[2]],
        [numblock[0],numblock[1]+4,numblock[2]+1],
        [numblock[0],numblock[1]+4,numblock[2]+2],
        [numblock[0],numblock[1]+3,numblock[2]],
        [numblock[0],numblock[1]+3,numblock[2]+2],
        [numblock[0],numblock[1]+2,numblock[2]],
        [numblock[0],numblock[1]+2,numblock[2]+1],
        [numblock[0],numblock[1]+2,numblock[2]+2],
        [numblock[0],numblock[1]+1,numblock[2]+2],
        numblock,
        [numblock[0],numblock[1],numblock[2]+1],
        [numblock[0],numblock[1],numblock[2]+2]
    ]
}
let targets = [
    { x: 75.50, y: 82, z: 105.52 },
    { x: 76.50, y: 75, z: 105.52 },
    { x: 72.50, y: 81, z: 105.52 },
    { x: 72.50, y: 74, z: 105.52 },
    { x: 69.50, y: 83, z: 105.52 },
    { x: 69.50, y: 78, z: 105.52 },
    { x: 67.50, y: 83, z: 105.52 },
    { x: 67.50, y: 75, z: 105.52 }
]


function setNumBlocks(player, num, correct = true){
    let block = "quartz_block";
    if(!correct) block = "red_concrete";
    num.forEach(blocks => {
       runCommand(player, `setblock ${blocks[0]} ${blocks[1]} ${blocks[2]} ${block}`); 
    });
}

function resetNums(player){
    runCommand(player, `fill ${numblock[0]} ${numblock[1]} ${numblock[2]} ${numblock[0]} ${numblock[1]+4} ${numblock[2]+2} polished_blackstone_bricks`);
}

function startRounds(player) {
    round += 1;

    roundId = system.runInterval(() => {
        runCommand(player, `title @s actionbar §lRound ${round}`);
    }, 40);

    resetNums(player);
    random = Math.floor(Math.random() * 10);

    distinctNumbers = [random];
    while (distinctNumbers.length < 3) {
        let newRandom = Math.floor(Math.random() * 10);
        if (!distinctNumbers.includes(newRandom)) {
            distinctNumbers.push(newRandom);
        }
    }
    for (let i = distinctNumbers.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        [distinctNumbers[i], distinctNumbers[j]] = [distinctNumbers[j], distinctNumbers[i]];
    }

    let inv = player.getComponent('inventory').container;

    let reddye = new ItemStack(ItemTypes.get("minecraft:red_dye"), 1);
    let bluedye = new ItemStack(ItemTypes.get("minecraft:blue_dye"), 1);
    let yellowdye = new ItemStack(ItemTypes.get("minecraft:yellow_dye"), 1);
    reddye.lockMode = ItemLockMode.slot;
    bluedye.lockMode = ItemLockMode.slot;
    yellowdye.lockMode = ItemLockMode.slot;
    reddye.nameTag = `§c§lNumber ${distinctNumbers[0]}`;
    bluedye.nameTag = `§c§lNumber ${distinctNumbers[1]}`;
    yellowdye.nameTag = `§c§lNumber ${distinctNumbers[2]}`;
    reddye.setLore(["§c§lRight click or use to select this option."]);
    bluedye.setLore(["§c§lRight click or use to select this option."]);
    bluedye.setLore(["§c§lRight click or use to select this option."]);

    inv.setItem(3, reddye);
    inv.setItem(4, bluedye);
    inv.setItem(5, yellowdye);

}

function expressFeelings(sender, receiver, senderMsg, receiverMsg, globalMsg, feeling) {
    world.getDimension("overworld").runCommandAsync(`execute as ${receiver.name} at ${receiver.name} run function ${feeling}`);

    sender.sendMessage(senderMsg.replace("${sender.name}", sender.name).replace("${receiver.name}", receiver.name));
    receiver.sendMessage(receiverMsg.replace("${sender.name}", sender.name).replace("${receiver.name}", receiver.name));

    Array.from(world.getPlayers()).forEach((player) => {
        if (![sender.name, receiver.name].includes(player.name))
            player.sendMessage(globalMsg.replace("${sender.name}", sender.name).replace("${receiver.name}", receiver.name));
    });
}


function teleport(player, location) {
    system.run(() => {
        world.getDimension(player.dimension.id).runCommandAsync(`tp ${player.name} ${location[0]} ${location[1]} ${location[2]}`);
    });
}

function runCommand(player, command) {
    system.run(() => {
        world.getDimension("overworld").runCommandAsync(`execute as ${player.name} at ${player.name} run ${command}`);
    });
}


system.afterEvents.scriptEventReceive.subscribe((event) => {
    const player = event.sourceEntity;
    const message = event.message;
    const id = event.id;
    if (message == "water" && id == "ww:spleef" && gamestarted == true) {
        const plrIndx = participatedPlayers.indexOf(player);
        if (plrIndx == -1) {
            return;
        }
        runCommand(player, "tag @s remove playing");
        runCommand(player, "effect @s clear");
        const eliminated = participatedPlayers.splice(plrIndx, 1);
        eliminatedPlayers.push(eliminated[0]);
        teleport(player, mainlobby);
        runCommand(player, "clear @s");
        participatedPlayers.forEach((plr) => {
            plr.sendMessage(`§8§l> §r§f${player.name}§7 was eliminated.`);
        });
        eliminatedPlayers.forEach((plr) => {
            plr.sendMessage(`§8§l> §r§f${player.name}§7 was eliminated.`);
        });

        if (participatedPlayers.length == 1) {
            participatedPlayers[0].sendMessage("§8§l> §r§fCongratulations!§7 You are the winner.");
            eliminatedPlayers.forEach((plr) => {
                plr.sendMessage(`§8§l> §r§f${participatedPlayers[0].name}§7 is the winner.`);
            });
            teleport(participatedPlayers[0], mainlobby);
            runCommand(participatedPlayers[0], "clear @s");
            runCommand(participatedPlayers[0], "give @s ww:bell 16");
            runCommand(participatedPlayers[0], "function spleefreset");
            runCommand(participatedPlayers[0], "tag @s remove playing");
            runCommand(participatedPlayers[0], "effect @s clear");
            participatedPlayers = [];
            eliminatedPlayers = [];
            gamestarted = false;
        }
    }
    else if (id == "ww:target"){
        let barrel = world.getDimension("overworld").getBlock({x: 163,y: 70,z: 126}).getComponent("inventory").container;
        let reward = new ItemStack(ItemTypes.get("ww:bell"), 1);
        barrel.addItem(reward);
    }
    else if(message == "givegift"){
        const amount = Math.floor((Math.random() * 6) + 3);
        runCommand(player, `give @s ${id} ${amount} 0 {"minecraft:can_place_on":{"blocks":["deepslate_bricks"]}}`);
        return;
    }
    else if (id == "ww:time"){
        let time = 18000;
        if(timePeriod == "night"){
            time = 1000;
            timePeriod = "day";
        }
        else{
            time = 18000;
            timePeriod = "night";
        }

        world.setTimeOfDay(time);

    }
});

world.beforeEvents.playerInteractWithBlock.subscribe((event) => {   
    let block = event.block;
    if(block.typeId == "minecraft:birch_trapdoor"){
        event.cancel = true;
    }
    if(block.typeId == "minecraft:hopper"){
        event.cancel = true;
    }
    if(block.typeId == "minecraft:frame"){
        event.cancel = true;
    }
});

world.afterEvents.buttonPush.subscribe((buttonPushEvent) => {
    let eventLoc = buttonPushEvent.block.location;
    let player = buttonPushEvent.source;

    if((eventLoc.x == snowballrestock1[0] && eventLoc.z == snowballrestock1[1]) || (eventLoc.x == snowballrestock2[0] && eventLoc.z == snowballrestock2[1]) || (eventLoc.x == snowballrestock3[0] && eventLoc.z == snowballrestock3[1]) || (eventLoc.x == snowballrestock4[0] && eventLoc.z == snowballrestock4[1]) || (eventLoc.x == dunksnowball[0] && eventLoc.z == dunksnowball[2])){
        runCommand(player, "give @s snowball 32");
        return;
    }

    if(eventLoc.x == bowarrow[0] && eventLoc.z == bowarrow[1]){
        runCommand(player, "give @s bow");
        runCommand(player, "give @s arrow 16");
        return;
    }
    else if(eventLoc.x == startmap[0] && eventLoc.y == startmap[1] && eventLoc.z == startmap[2]){
        if(startedmapPlayers.indexOf(player) == -1){
            startedmapPlayers.push(player);
            timeline.start_scene(player);
        }
        else{
            player.sendMessage("§8§l> §r§7You have already started the map.");
        }
    }


    if (gamestarted) {
        player.sendMessage("§8§l> §r§7A game is already on the go. Please wait till it ends.");
        return;
    }
    if (eventLoc.x == joinbutton[0] && eventLoc.y == joinbutton[1] && eventLoc.z == joinbutton[2]) {
        if (participatedPlayers.indexOf(player) == -1) {
            player.sendMessage(`§8§l> §r§7Joining the lobby!`);
            system.runTimeout(() => {
                teleport(player, gamelobby);
                if (participatedPlayers.length == 2) {
                    participatedPlayers.forEach((plr) => {
                        plr.sendMessage("§8§l> §r§7Game can now begin! When all players are joined and ready, any one player press the start button.");
                    });
                }
                else {
                    player.sendMessage(`§8§l> §r§7Two or more players required. Waiting for sufficient players to join...`);
                }
            }, 40);
            participatedPlayers.push(player);
        }
    }
    else if (eventLoc.x == leavebutton[0] && eventLoc.y == leavebutton[1] && eventLoc.z == leavebutton[2]) {
        const plrIndx = participatedPlayers.indexOf(player);
        if (plrIndx == -1) {
            return;
        }
        player.sendMessage(`§8§l> §r§7Exiting the lobby!`);
        participatedPlayers.splice(plrIndx, 1);
        system.runTimeout(() => {
            teleport(player, mainlobby);
        }, 20);
    }
    else if (eventLoc.x == startspleefbutton[0] && eventLoc.y == startspleefbutton[1] && eventLoc.z == startspleefbutton[2]) {
        if(gamestarted == true){
            return;
        }
        const plrIndx = participatedPlayers.indexOf(player);
        if (plrIndx == -1) {
            return;
        }
        if (participatedPlayers.length >= 2) {
            let i = 0;
            gamestarted = true;
            const intervalId = system.runInterval(() => {
                if (i < 6) {
                    participatedPlayers.forEach((plr) => {
                        runCommand(plr, `title @a actionbar Starting game in ${5 - i} seconds...`);
                    });

                    i++;
                } else {
                    system.clearRun(intervalId);

                    participatedPlayers.forEach((plr) => {
                        teleport(plr, spawn);
                        runCommand(plr, "function startspleefgame");
                        runCommand(plr, "tag @s add playing");
                        runCommand(plr, "effect @s resistance 999999 3 true");
                    });
                }
            }, 20);
        }

        else {
            player.sendMessage("§8§l> §r§7Not sufficient players for the game. Waiting for more players to join...");
        }
    }
    else if(eventLoc.x == startdunkbutton[0] && eventLoc.z == startdunkbutton[2]){
        if(dunksSpawned != 0){
            player.sendMessage("§8§l> §r§7Some elves are still on the platform. Dunk them to start another game.");
            return;
        }
        dunkPlayer = player
        player.runCommandAsync("give @s snowball 16");
        targets.forEach((target)=>{
            overworld.spawnEntity("ww:dunk_tank_elf", target);
            dunksSpawned += 1;
        });
    }
    else if (eventLoc.x == groundbutton[0] && eventLoc.y == groundbutton[1] && eventLoc.z == groundbutton[2]) {
        system.runTimeout(() => {
            player.sendMessage("§8§l> §r§7Teleporting to ground...");
            teleport(player, ground);
        }, 20);
    }
    else if (eventLoc.x == startguessbutton[0] && eventLoc.z == startguessbutton[2] && guessStarted == false) {
        guessStarted = true;
        player.sendMessage("§8§l> §r§7Teleporting to platform...");
        runCommand(player, "clear @s");
        system.runTimeout(() => {
            teleport(player, platform);
            startRounds(player);
        }, 40);
        return;
    }
    else if (eventLoc.x == startguessbutton[0] && eventLoc.z == startguessbutton[2] && guessStarted == true) {
        player.sendMessage("§8§l> §r§7Game is already going on.. wait for some time.");
        return;
    }
    else if(eventLoc.x == coasterstart[0] && eventLoc.z == coasterstart[2]){
        let component = player.getComponent("minecraft:riding");
        if(!component){
            return;
        }
        else if(component.entityRidingOn.typeId != "minecraft:minecart"){
            return;
        }

        player.playSound("note.bell");
        player.sendMessage("§8§l> §r§7Enjoy the Ride!");
        runCommand(player, "playsound coaster @s");
        return;
    }
});

function handleBasicConfig(args, sender) {
    if (args.length === 0) {
        showHelp(sender);
        return;
    }
    switch (args[0]) {
        case "help":
            showHelp(sender);
            break;
        default:
            sender.sendMessage("§l§cInvalid Comand! §fUse \"!cf help\" to view the commands.");
            break;
    }
}

function open_menu(player) {

    const form = new ActionFormData().title("§k--------§r§l§5]-Map Menu-[§r§k--------");
    places.forEach((place) => {
        form.button("§l§c" + place.name, place.icon);
    });

    form.show(player).then((response) => {
        if (response.canceled || response.selection == undefined) {
            return;
        }
        let index = Number(response.selection);

        teleport(player, places[index].coords);

    });
}


beforeEvents.chatSend.subscribe((data) => {

    if (!data.message.startsWith("!")) return;

    const sender = data.sender;
    data.cancel = true;

    const args = data.message.split(" ");
    const command = args.shift().substring(1);

    if (command === "cf") {
        handleBasicConfig(args, sender);
        return;
    }
    else if (command === "feelings") {
        showFeelings(args, sender);
        return;
    }
    else if (command === "clearchat") {
        for (let i = 0; i < 30; i++) {
            sender.sendMessage(" ");
        }
        return;
    }

    const feelingObj = feelings.find(feeling => feeling.command === command);
    if (feelingObj) {
        const receiver = getPlayerByName(args[0]);
        if (!receiver) {
            sender.sendMessage(`§c§lPlayer ${args[0]} doesn't exists!`);
            return;
        }

        expressFeelings(
            sender,
            receiver,
            feelingObj.senderMsg,
            feelingObj.receivermsg,
            feelingObj.globalMsg,
            feelingObj.feeling
        );
    }
    else {
        sender.sendMessage("\n§l§aChatFeelings §r§8|§r §l§cInvalid Command! Use §f!cf help§c for more information.§r");
    }

});

function restartRound(player){
    guessStarted = true;
    startRounds(player);
}

afterEvents.itemUse.subscribe((data) => {
    const playerName = data.source.name;
    const player = getPlayerByName(playerName);
    if (player && data.itemStack.typeId?.toLowerCase() === listItem) {
        open_menu(player);
    }
    else if (guessStarted==true) {
        const dye = data.itemStack.typeId?.toLowerCase();
        if(dye != "minecraft:red_dye" && dye != "minecraft:blue_dye" && dye != "minecraft:yellow_dye") return;
        let correct = false;
        let number = 0;
        runCommand(player, "clear @s");

        if(dye == "minecraft:red_dye"){
            if(distinctNumbers[0] != random){
                correct = true;
                number = distinctNumbers[0];
            }
            else{
                number = distinctNumbers[1];
            }
        }
        else if(dye == "minecraft:blue_dye"){
            if(distinctNumbers[1] != random){
                correct = true;
                number = distinctNumbers[1];
            }
            else{
                number = distinctNumbers[2];
            }
        }
        else if(dye == "minecraft:yellow_dye"){
            if(distinctNumbers[2] != random){
                correct = true;
                number = distinctNumbers[2];
            }
            else{
                number = distinctNumbers[0];
            }
        }

        setNumBlocks(player, num[number.toString()], correct);

        if(correct){
            player.sendMessage("§8§l> §r§7You guessed the number correct");
            score += 1;
            runCommand(player, "playsound note.bell @s");
        }
        else{
            player.sendMessage("§8§l> §r§7You guessed the number incorrect");
            runCommand(player, "playsound note.didgeridoo @s");
        }

        system.clearRun(roundId);
        runCommand(player, "clear @s");
    
        if(round < 5){
            let count = 6;
            let id = system.runInterval(()=>{
                count -= 1;
                runCommand(player, `title @s actionbar Starting next round in ${count}`);
                if(count == 0){
                    system.clearRun(id);
                    restartRound(player);
                }
            },20);
        }
        else{
            guessStarted = false;
            resetNums(player);
            player.sendMessage(`§8§l> §r§fYou scored ${score} out of 5.`);
            teleport(player, startguessbutton);
            let amount = Math.pow(2, score);
            runCommand(player, `give @s ww:bell ${amount}`);
            score = 0;
            round = 0;
            roundId = null;
            return;
        }
        
    }
});

afterEvents.entityRemove.subscribe((data)=>{
    let entity = data.typeId;
    if(entity == "ww:dunk_tank_elf"){
        dunksSpawned -= 1;
        dunkPlayer.playSound("note.bell");
        dunkPlayer.runCommandAsync("loot spawn 71 70 98 loot dunk");
    }
});

afterEvents.pressurePlatePush.subscribe((event)=>{
    const player = event.source;
    const uplocations = [
        [-19, 86, -82],
        [-8, 95, -92],
        [5, 105, -103],
        [14, 116, -115],
        [26, 126, -128],    
        [39, 134, -138],
        [50, 140, -150],
        [61, 142, -159]
    ];
    const downlocations = [
        [55, 139, -145],
        [44, 133, -133],
        [31, 125, -123],
        [19, 113, -109],
        [9, 103, -97],    
        [-2, 94, -87],
        [-13, 85, -78],
        [-18, 75, -66]
    ];
    let upelevate = true;
    
    if(player.location.y >= 140){
        upelevate = false;
    }
    else if(player.location.y == 76){
        upelevate = true
    }
    else{
        return;
    }
    player.sendMessage("§8§l> §r§fStarting elevator..");
    let i = 0;
    let intid = system.runInterval(()=>{
        if(i == 0){
            player.playSound("note.chime");
        }
        if(upelevate){
            teleport(player, uplocations[i]);
        }
        else{
            teleport(player, downlocations[i]);
        }
        if(i == 7){
            player.playSound("note.chime");
            system.clearRun(intid);
            return;
        }
        i+=1;
    },20);
});


afterEvents.playerInteractWithEntity.subscribe((event)=>{
    let player = event.player;
    let entity = event.target;
    let location = entity.location;
    const x = (location.x > 0) ? Math.ceil(location.x) : Math.floor(location.x);
    const z = (location.z > 0) ? Math.ceil(location.z) : Math.floor(location.z);

    if(entity.typeId == guide){
        interactable_entities.forEach((guide)=>{
            if(Math.abs(x - guide.location.x) <= 1 && Math.abs(z - guide.location.z) <= 1){
                guide.time.forEach((time, index)=>{
                    system.runTimeout(()=>{
                        player.sendMessage("§r§l§c" + guide.name + "§r" + guide.dialogue[index]);
                        player.playSound("note.bell");
                    },time);
                });
                player.playSound(guide.sound);
                return;
            }
        });
    }
});